/* *****************************************************************************
 * decode/mcs/program.c -- decode an MCS program.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libg1m.
 * libg1m is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libg1m is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libg1m; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libg1m/internals.h>

/**
 *	g1m_decode_mcs_program:
 *	Decode a program.
 *
 *	@arg	handle		the handle.
 *	@arg	buffer		the buffer to read from.
 *	@arg	length		the data length.
 *	@return				the error code (0 if ok).
 */

int g1m_decode_mcs_program(g1m_mcsfile_t *handle, g1m_buffer_t *buffer,
	uint_fast32_t length)
{
	/* read header */
	DREAD(hd, mcs_programheader)

	/* print header data */
	log_info("Program password is '%.8s'.", hd.password);

	/* store info */
	strncpy(handle->password, (char*)hd.password, 8);
	handle->password[8] = 0;

	/* get content */
	size_t content_size = length - sizeof(struct mcs_programheader);
	log_info("Getting program content (%" PRIuSIZE "o)", content_size);
	handle->content = malloc(content_size + 1);
	handle->head.size = content_size + 1;
	if (!handle->content) return (g1m_error_alloc);
	READ(handle->content, content_size);
	handle->content[content_size] = 0;

	/* no error */
	return (0);
}
