/* *****************************************************************************
 * utils/membuffer.c -- memory to libg1m buffer utilities.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libg1m.
 * libg1m is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libg1m is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libg1m; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libg1m/internals.h>
#include <string.h>

/**
 *	g1m_membuffer_read:
 *	Read from a memory buffer.
 *
 *	@arg	vcookie		the cursor (uncasted)
 *	@arg	dest		the destination buffer.
 *	@arg	size		the size to read.
 *	@return				the error, if any.
 */

int g1m_membuffer_read(void *vcookie, unsigned char *dest, size_t size)
{
	g1m_cursor_t *cursor = (void*)vcookie;

	if (size > cursor->left)
		return (g1m_error_eof);
	memcpy(dest, cursor->p, size);
	cursor->p += size;
	return (0);
}
